/******************************************************************************
 uart_statemachine.c - uart state machine source file for communication
 Version: 0.9.1
 Created on: 21/05/2021
 Created by: Rohit Rajapure

 Copyright (c) 2017 Shalaka Connected Devices LLP.  All rights reserved.
 Software License Agreement

   Redistribution and use in source and binary forms, with or without
   modification, are not permitted.

   Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

   Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the
   distribution.

   Neither the name of Shalaka Connected Devices LLP nor the names of
   its contributors may be used to endorse or promote products derived
   from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ******************************************************************************/

#include <stdbool.h>
#include <stdint.h>
#include <stdio.h>
#include "stdlib.h"
#include "string.h"
#include "zigbee_helpers.h"
#include "app_uart.h"
#include "nrf_drv_uart.h"
#include "app_error.h"
#include "nrf_delay.h"
#include "nrf.h"

#include "uart_config.h"
#include "gpio_config.h"
#include "timer_config.h"
#include "uart_statemachine.h" 
#include "commands.h"
#include "app_rtc.h"
#include "zb_custom1_thermostat.h"
#include "custom_app_event.h"

/*******************************************************************************************
*MACROS
*
*******************************************************************************************/

/******************************************************************************************
*TYPEDEFS
*
*******************************************************************************************/
State CurrentState, PreviousState;

State CurrentState = State_Ready;

E_StateMachine e_StateMachine;
E_StateMachineThermostat e_StateMachineThermostat;

E_StateMachine e_StateMachine = IDLE_STATE;
E_StateMachineThermostat e_StateMachineThermostat = POWER_CYCLE;

App_tsEventUart sAppUartEvent;
App_tsEventBootloader sAppBootloaderEvent;
App_tsEventProduction sAppProductionEvent;

/******************************************************************************************
*EXTERNAL VARIABLES
*
*******************************************************************************************/

extern bool boolReady;
extern bool boolSetTemperature;
extern uint8_t ui8GetCommandBuffer[BUFFER_SIZE], ui8SetCommandBuffer[BUFFER_SIZE],ui8SendDeviceConfig[BUFFER_SIZE], ui8GetFlagBuffer[BUFFER_SIZE], ui8ResponceValueSend[BUFFER_SIZE], ui8ResponceValueSend2[BUFFER_SIZE], ui8StartFirmwareUpdate[BUFFER_SIZE], ui8TransferFirmware[BUFFER_SIZE], ui8TempGetFlagBuffer[BUFFER_SIZE];
extern bool boolGetStatus;
extern bool boolSetHostflags;
extern uint32_t ui32HostflagsData;
extern bool boolSendDeviceConfig;
/******************************************************************************************
*LOCAL VARIABLES
*
*******************************************************************************************/
uint8_t ui8ReadGetFlagBuffer[BUFFER_SIZE] = {0};
uint8_t ui8SetpointChange[SET_POINT_CHANGE_BUFFER] = {0}; //store set point change value into buffer
uint8_t ui8BatteryStatus[4] = {0};
uint8_t ui8ValvePositionChange[2] = {0};
uint8_t ui8FirmwareVersion[26] = {0};
uint8_t ui8DeviceConfigData[6] = {0};
uint8_t ui8GetBootloaderStatus[2] = {0};
uint8_t ui8Rx = 0, ui8RxIndex =0, ui8Index =0;
uint16_t ui16LocalTemperature = 0, ui16SetPointChange = 0, ui16UnOccupiedTemp = 0, ui16OccupiedTemp = 0;
uint8_t ui8TelegramRepeatCount = 0;
uint32_t ui32ReadHostFlagsByte = 0;
uint32_t ui32CurrentVersion[6] = {0};

char u8CurrentRFVersionString[] = "";
char u32CurrentVersionString[] = "";

uint16_t  ui16BatteryVoltage = 0;
float flBatteryPercentage = 0.0;


uint16_t ui16SaveAttributeID;
uint8_t ui8TrvModeBackup;
bool StartFrimwareUpdateTimer = false;

static uint32_t ui32HostImageStartAddress;
static uint32_t ui32HostImageEndAddress;
bool boolSetCommand = false;
bool boolIsMCURequest = false; //this flag is general to send getflag command whenever requried. So, set this flag accordingly.
bool boolMCUInterrupt = false;
bool FirmwareUpdateTransferFinished  = false;
/******************************************************************************************
*BOOLEAN VARIABLES FOR GETFLAG COMMANDS
*
*******************************************************************************************/
//Data 01
bool boolManualMode = false; 
bool boolHeatingMode = false;
bool boolKeyProtection = false;
bool boolInternalWindowOpen = false;
bool boolExternalWindowOpen = false;
bool boolRTCInValid = false;
bool boolBatteryWeak = false;
bool boolHolidayMode = false;
//Data 02
bool boolEncoderChange = false;
bool boolSetPointChange = false;
bool boolMasterResetRequest = false;
bool boolMeasuredTempChange = false;
bool boolMasterResetChange = false;
bool boolBoostModeActive = false;
bool boolProductModeRF = false;
bool boolValvePositionChange = false;
//Data 03
bool boolButton1Push = false;
bool boolButton2Push = false;
bool boolButton3Push = false;
bool boolButton4Push = false;
bool boolButton5Push = false;
bool boolButtonShortPush = false;
bool boolButtonLongPush = false;
bool boolButtonVeryLongpush = false;
//Data 04
bool boolADAPSuccess = false;
bool boolErrorE1State = false;
bool boolErrorE2State = false;
bool boolErrorE3State = false;
bool boolPrepMode = false;
bool boolINST = false;
bool boolADAP = false;
bool boolADAPNotValid = false;
//Data 05
bool boolPairRequest = false;
bool boolUnPairRequest = false;
bool boolPairTimeOut = false;
bool boolDevicePairState = false;

/******************************************************************************************
*FUNCTIONS
*
*******************************************************************************************/


/******************************************************************************************
*function for communication state handling between MCU and Network processor 
*(this function used to execute the state machine between Radiator and nRF node)
*return void
*******************************************************************************************/

void CommunicationStateExecution(void)
{
   if(boolReady)
   {
       switch(CurrentState)
       {
        case State_Ready:         
                Uart_Init_Func(); 
                PreviousState = CurrentState;   //if interrupt on ULP switch state to command
                CurrentState  = State_Command;
                ui8TelegramRepeatCount =0;
        break;

        case State_Command:             
              ulp_pin_transaction();  //ULP pin toggle configuration and high for 5msec
              //send get flag telegram message to device 
              if(!boolIsMCURequest)
              {
                 GetMessageIdResponse(GET_FLAGS); 
                
              }
             
              Uart_PutData(ui8GetCommandBuffer,sizeof(ui8GetCommandBuffer));   
              memset(ui8GetCommandBuffer,0,sizeof(ui8GetCommandBuffer)); 

              PreviousState = CurrentState;   //after command sent switch state to read ACK
              CurrentState  = State_ACKRead;
        break;

        case State_ACKRead: 
              //after get flag read ACK and send EOC 
              Uart_GetByte(&ui8Rx);
              if((ui8Rx ==RESP_ACK) && (boolSetCommand == true))
              {                                       
                  ui8TelegramRepeatCount =0;
                  boolSetCommand = false;
                  PreviousState = CurrentState;   //if EOC and Set command then switch to state Set_Command
                  CurrentState  = State_SetCommand;
              }
              else if((ui8Rx ==RESP_ACK) && (boolSetTemperature == true))
              {                            
                  ui8TelegramRepeatCount =0;                  
                  PreviousState = CurrentState;   //if EOC and Set command then switch to state Set_Command
                  CurrentState  = State_SetCommand;
              }
              else if((ui8Rx ==RESP_ACK) && (boolSetHostflags == true))
              {              
                ui8TelegramRepeatCount =0;                       
                PreviousState = CurrentState;   //if EOC and Set command then switch to state Set_Command
                CurrentState  = State_SetCommand;
              }
              else if((ui8Rx ==RESP_ACK) && (boolSendDeviceConfig == true))
              {                
                ui8TelegramRepeatCount =0;                       
                PreviousState = CurrentState;   //if EOC and Set command then switch to state Set_Command
                CurrentState  = State_SetCommand;
              }
              else if(ui8Rx ==RESP_ACK)
              {                                
                ui8TelegramRepeatCount =0;                       
                PreviousState = CurrentState;   //after EOC read switch state to send EOC 
                CurrentState  = State_SendEOC;
              }
              //if statement to check NACK response and perform the repeatation action 
              if(ui8Rx == RESP_NACK)
              {
                 ui8TelegramRepeatCount++;

                 if(ui8TelegramRepeatCount == 3)
                 {
                     PreviousState = CurrentState;   //After command repeatation is done for 3 times switch state to Ready state
                     CurrentState  = State_Ready;
                     ui8TelegramRepeatCount =0;
                     boolReady = false;
                 }
                 else
                 {
                     //if NACK then repeat the command for 3 time.
                     CurrentState  = PreviousState;
                 }
              }

              if(ui8Rx == RESP_TNS)
              {
                   PreviousState = CurrentState;   //After command repeatation if it not supportive command (TNS) then go to ready state
                   CurrentState  = State_Ready;
                   boolReady = false;
              }
        break;

        case State_SendEOC:
              Uart_PutByte(EOC);
              
              PreviousState = CurrentState;   //after EOC read switch state to read dummy ACK
              CurrentState  = State_ACKReadDummy;
        break;

        case State_ACKReadDummy: 
              //after get flag read ACK and send EOC 
              Uart_GetByte(&ui8Rx);
              if(ui8Rx ==RESP_ACK)
              {                       
                ui8TelegramRepeatCount =0;
                PreviousState = CurrentState;   //after dummy ACK read switch state to read response data 
                CurrentState  = State_ReadDataResponse;             
              }
        break;

        case State_SetCommand:
             if((!boolSetTemperature) && (!boolSetHostflags) && (!boolSendDeviceConfig))
             {
               SetFlags();  //send set flag command
               
               Uart_PutData(ui8SetCommandBuffer,sizeof(ui8SetCommandBuffer));
               memset(ui8SetCommandBuffer,0,sizeof(ui8SetCommandBuffer)); 
             }
             else if((boolSetTemperature) && (!boolSetHostflags) && (!boolSendDeviceConfig))
             {
               SendSetTemperature();
              
               boolSetTemperature = false;   
               Uart_PutData(ui8SetCommandBuffer,sizeof(ui8SetCommandBuffer));
               memset(ui8SetCommandBuffer,0,sizeof(ui8SetCommandBuffer)); 
             }
             else if(boolSetHostflags)
             {
               UARTStateSendFlags();  
               boolSetHostflags = false;
               
               Uart_PutData(ui8SetCommandBuffer,sizeof(ui8SetCommandBuffer));
               memset(ui8SetCommandBuffer,0,sizeof(ui8SetCommandBuffer)); 
             }
             else if(boolSendDeviceConfig)
             {
                 UARTStateSendDeviceConfig();          //send set device config command
                 boolSendDeviceConfig = false;
                
                 Uart_PutData(ui8SendDeviceConfig,sizeof(ui8SendDeviceConfig));
                 memset(ui8SendDeviceConfig,0,sizeof(ui8SendDeviceConfig)); 
             }            
              PreviousState = CurrentState;   //after command sent switch state to read ACK
              CurrentState  = State_ACKRead;
         break;

        case State_ReadDataResponse: 
              //read command response from MCU             
              Uart_GetByte(&ui8Rx);
              if(ui8Rx == SYNC)
              {                
                 //read length of data
                 Uart_GetByte(&ui8Rx);
                 ui8RxIndex = ui8Rx;

                 GetFlagsResponse();   //store getflag data into buffer

                // after data read send ACK to device 
                 Uart_PutByte(RESP_ACK);

                 PreviousState = CurrentState;   //after data read send ACK to device  switch state EOC  
                 CurrentState  = State_EOCRead;

              }
        break;

        case State_EOCRead: 
             //After data read from MCU check for EOC
              Uart_GetByte(&ui8Rx);
              if(ui8Rx ==EOC)
              {                      
                //send ACK back
                  Uart_PutByte(RESP_ACK);
                  PreviousState = CurrentState;   //all state executed then switch to ready state
                  CurrentState  = State_EndCommunication; 
//                  boolReady = false; 
//                  boolMCUInterrupt = true;   
              }              
              
        break;

        case State_EndCommunication:                
                boolReady = false; 
                boolMCUInterrupt = true;   
                PreviousState = CurrentState;   //all state executed then switch to ready state
                CurrentState  = State_Ready;
                ResetFlags();                                
        break;

        default:
        break;

         
       }
   }

}

/******************************************************************************************
*function for read the getflag bits and store into variable
*return void
*******************************************************************************************/
void GetFlagsResponse (void)
{
     for(ui8Index = 0; ui8Index< ui8RxIndex-2; ui8Index++)  //loop to store read get flag data into variable
     {
        //store data into buffer
        Uart_GetByte(&ui8Rx);
        ui8ReadGetFlagBuffer[ui8Index] = ui8Rx;
     }
     
}


/******************************************************************************************
*This function is used to process the commands based on Id comparision.
*return void
*******************************************************************************************/
void DataProcessing(void)
{
   if(boolMCUInterrupt)
   {
       switch(ui8ReadGetFlagBuffer[0])
       {
          case GET_FLAGS:
               memcpy(ui8GetFlagBuffer,ui8ReadGetFlagBuffer,sizeof(ui8ReadGetFlagBuffer));  
               nrf_delay_ms(25);                         

                //Data 01 of flag byte
               if(ui8GetFlagBuffer[1] & FLAGS_BYTE_1_MANU_MODE)
               {
//                  SetTRVMode(ZB_ZCL_THERMOSTAT_TRV_MODE_DEFAULT_VALUE);
                  m_dev_ctx.thermostat_attr.trv_mode = ZB_ZCL_THERMOSTAT_TRV_MODE_DEFAULT_VALUE;
                  m_dev_ctx.thermostat_attr.host_flags |= FLAGS_BYTE_1_MANU_MODE;
//                  HostFlagStatus(ui32ReadHostFlagsByte);
               }

               if(ui8GetFlagBuffer[1] & FLAGS_BYTE_1_KEY_PROTECTION_ACTIVE)
               {
                 m_dev_ctx.thermostat_attr.host_flags |= 0x000080;
//                 HostFlagStatus(ui32ReadHostFlagsByte);
               }
               if((ui8GetFlagBuffer[1] ^ FLAGS_BYTE_1_KEY_PROTECTION_ACTIVE)& FLAGS_BYTE_1_KEY_PROTECTION_ACTIVE)
               {
                 m_dev_ctx.thermostat_attr.host_flags &=m_dev_ctx.thermostat_attr.host_flags ^ 0x000080;
//                 HostFlagStatus(ui32ReadHostFlagsByte);
               }

               if(ui8GetFlagBuffer[1] & FLAGS_BYTE_1_INT_WINDOW_TRIGGERD)
               {
                  m_dev_ctx.thermostat_attr.host_flags |= START_INTERNAL_WINDOW_OPEN;
//                  HostFlagStatus(ui32ReadHostFlagsByte);
               }
              if ((ui8GetFlagBuffer[1] ^ FLAGS_BYTE_1_INT_WINDOW_TRIGGERD)& FLAGS_BYTE_1_INT_WINDOW_TRIGGERD) 
              {
                  m_dev_ctx.thermostat_attr.host_flags &= m_dev_ctx.thermostat_attr.host_flags ^ STOP_INTERNAL_WINDOW_OPEN;
//                  HostFlagStatus(ui32ReadHostFlagsByte);
              }
              
              if (ui8GetFlagBuffer[1] & FLAGS_BYTE_1_EXT_WINDOW_ACTIVATE) 
              {      
              // If the external window open detection is active, the same bit is set in the RF telegram as for the internal one.
                 m_dev_ctx.thermostat_attr.host_flags |= START_INTERNAL_WINDOW_OPEN;
//                 HostFlagStatus(ui32ReadHostFlagsByte);
              }

              if (ui8GetFlagBuffer[1] & FLAGS_BYTE_1_BATTERY_WEAK) 
              {  
//                  ReadErrorStatus(BATTERY_WEAK_ERROR);
                  m_dev_ctx.thermostat_attr.error |= 0x01;
                  boolReady = true; //set flag to execute communication state
                  boolIsMCURequest = true;
                  e_StateMachineThermostat = RUNNING_STATE;
                  sAppUartEvent.eType_UartNextState = APP_E_EVENT_UART_GET_BATTERY;
              }

              if((ui8GetFlagBuffer[1] ^ FLAGS_BYTE_1_BATTERY_WEAK) & FLAGS_BYTE_1_BATTERY_WEAK )
              {                
//                  ui32ReadHostFlagsByte &= ui32ReadHostFlagsByte ^ 0x01;
//                  ReadErrorStatus(0);
                  m_dev_ctx.thermostat_attr.error &=  m_dev_ctx.thermostat_attr.error ^ 0x01;
              }

               //Data 02 of flag byte
               //Query whether setpoint temperature has been changed
              if (ui8GetFlagBuffer[2] & FLAGS_BYTE_2_SETPOINT_CHANGED) 
              {
                  boolReady = true; //set flag to execute communication state
                  boolIsMCURequest = true;        
                  e_StateMachineThermostat = RUNNING_STATE;
                  sAppUartEvent.eType_UartNextState = APP_E_EVENT_UART_GET_TEMPERATURE;
              }
              
              //Master reset is executed
              if(ui8GetFlagBuffer[2] & FLAGS_BYTE_2_MASTER_RESET_REQUEST)
              {
                 //Erase bonding information 
                 zigbee_erase_persistent_storage(ZB_TRUE);
                 //soft reset 
                 NVIC_SystemReset(); 
                 
              }
              //Has the ACTUAL temperature changed, a request is sent to the controller.
              if(ui8GetFlagBuffer[2] & FLAGS_BYTE_2_MEASURED_TEMP_CHANGED)
              {
                  boolReady = true; //set flag to execute communication state
                  boolIsMCURequest = true;        
                  e_StateMachineThermostat = RUNNING_STATE;
                  sAppUartEvent.eType_UartNextState = APP_E_EVENT_UART_GET_TEMPERATURE;
              }

              if(ui8GetFlagBuffer[2] & FLAGS_BYTE_2_BOOST_MODE)
              {
                m_dev_ctx.thermostat_attr.host_flags |= 0x04;
//                HostFlagStatus(ui32ReadHostFlagsByte);
              }

              if((ui8GetFlagBuffer[2] ^ FLAGS_BYTE_2_BOOST_MODE) & FLAGS_BYTE_2_BOOST_MODE) 
              {
                m_dev_ctx.thermostat_attr.host_flags &= m_dev_ctx.thermostat_attr.host_flags ^ 0x04; 
//                HostFlagStatus(ui32ReadHostFlagsByte);       
              }

              if (ui8GetFlagBuffer[2] & FLAGS_BYTE_2_VALVE_POSITION_CHANGED)
              {
                 boolReady = true; //set flag to execute communication state
                 boolIsMCURequest = true;
                 e_StateMachineThermostat = RUNNING_STATE;
                 sAppUartEvent.eType_UartNextState  = APP_E_EVENT_UART_GET_VALVE_POSITION;
              }

              //Data 04 flag byte
              if(ui8GetFlagBuffer[4] == FLAGS_BYTE_4_ADAP_SUCCESSFUL)
              {
                ReadErrorStatus(NO_ERROR);
              }
              if((ui8GetFlagBuffer[4] & (0x07)) == FLAGS_BYTE_4_ERROR_E1)
              {
                ReadErrorStatus(VALVE_ADAPTATION_FAIL_E1);//Error ER1 Adaptation failed because not mounted
              }
              if((ui8GetFlagBuffer[4] & (0x07)) == FLAGS_BYTE_4_ERROR_E2)
              {
                ReadErrorStatus(VALVE_MOVEMENT_SLOW_E2); //Error ER2 adaptation failed because closing point not found
              }
              if((ui8GetFlagBuffer[4] & (0x07)) == FLAGS_BYTE_4_ERROR_E3)
              {
                ReadErrorStatus(VALVE_NOT_MOVING_E3); //Error ER3 adaptation failed because valve stroke too short or adaptation not possible
              }

          break;

          case GET_BOOTLOADER_STATE:
               memcpy(ui8GetBootloaderStatus,ui8ReadGetFlagBuffer,sizeof(ui8GetBootloaderStatus));

               if(ui8GetBootloaderStatus[1] == 0x01)
               {
                  e_StateMachineThermostat = BOOTLOADER_STATE;
                  sAppBootloaderEvent.eType_BootloaderState = APP_E_EVENT_UART_START_FIRMWARE_UPDATE;
//                  sAppBootloaderEvent.eType_BootloaderState = APP_E_EVENT_UART_TRANSFER_FRIMWARE_UPDATE;
               }
               else if(ui8GetBootloaderStatus[1] == 0x00)
               {
                  e_StateMachineThermostat = RUNNING_STATE;
                  sAppUartEvent.eType_UartNextState = APP_E_EVENT_UART_GET_FIRMWARE_VERSION; // this state should execute only once when device starts 
               }
                boolIsMCURequest = false;
          break;

          case GET_TEMPERATURES:
               memcpy(ui8SetpointChange,ui8ReadGetFlagBuffer,sizeof(ui8SetpointChange));
              
              ui16LocalTemperature = ui8SetpointChange[1] << 8;
              ui16LocalTemperature = ui16LocalTemperature + ui8SetpointChange[2];
              localTemperatureValue(ui16LocalTemperature);

              ui16SetPointChange   = ui8SetpointChange[5] << 8;
              ui16SetPointChange   = ui16SetPointChange + ui8SetpointChange[6];              
              CurrentSetpointChangeValue(ui16SetPointChange);        
              
              ReadOccupiedHeatingSetpoint(ui16SetPointChange);  // When get temperature in requested copy the set point value on Occupied heating point id. 

              boolIsMCURequest = false;
              nrf_delay_ms(25);
              e_StateMachineThermostat = RUNNING_STATE;
              sAppUartEvent.eType_UartNextState = APP_E_EVENT_UART_GET_BATTERY;

          break;

          case GET_BATTERY_STATUS:
               memcpy(ui8BatteryStatus, ui8ReadGetFlagBuffer,sizeof(ui8BatteryStatus));
               //flBatteryPercentage = ((ui8BatteryStatus[1] * 100)/255);
               vCalculateBattery(ui8BatteryStatus[1]);

//               ui16BatteryVoltage = ui8BatteryStatus[2] << 8;    /**voltage calculation**/
//               ui16BatteryVoltage = ui16BatteryVoltage + ui8BatteryStatus[3];
              
               boolIsMCURequest = false;
               nrf_delay_ms(25);
                if(boolGetStatus)
                {
                    boolGetStatus = false;
                    e_StateMachineThermostat = RUNNING_STATE;
                    sAppUartEvent.eType_UartNextState = APP_E_EVENT_UART_GET_DEVICE_CONFIG; 
                }
                else
                {
                    sAppUartEvent.eType_UartNextState = APP_E_EVENT_UART_IDLE;
                }
          break;

          case GET_FIRMWARE_VER:
               memcpy(ui8FirmwareVersion,ui8ReadGetFlagBuffer,sizeof(ui8FirmwareVersion));
               ui32CurrentVersion[0] = (ui8FirmwareVersion[19] - 0x30)	* 100000; 
               ui32CurrentVersion[1] = (ui8FirmwareVersion[20] - 0x30)	* 10000; 
               ui32CurrentVersion[2] = (ui8FirmwareVersion[21] - 0x30)	* 1000; 
               ui32CurrentVersion[3] = (ui8FirmwareVersion[22] - 0x30)	* 100; 
               ui32CurrentVersion[4] = (ui8FirmwareVersion[23] - 0x30)	* 10; 
               ui32CurrentVersion[5] = (ui8FirmwareVersion[24] - 0x30); 
                
//               itoa(m_dev_ctx.basic_attr.app_version,u8CurrentRFVersionString,8);
//               itoa((int*)ui32CurrentVersion,u32CurrentVersionString,8);

//               strcat(u8CurrentRFVersionString, u32CurrentVersionString);
//
//               memcpy(m_dev_ctx.basic_attr.sw_ver,u8CurrentRFVersionString, 8);

               boolIsMCURequest = false; 
               nrf_delay_ms(25);
               e_StateMachineThermostat = RUNNING_STATE;
               sAppUartEvent.eType_UartNextState = APP_E_EVENT_UART_GET_DEVICE_CONFIG;
          break;

          case GET_DEVICE_CONFIG:
               memcpy(ui8DeviceConfigData,ui8ReadGetFlagBuffer,sizeof(ui8DeviceConfigData));
               
               if(ui8DeviceConfigData[3] == DEVICE_CONFIG_BYTE_3_VALVE_CONTROLLER)
               {
                  m_dev_ctx.thermostat_attr.trv_mode = 0x01;     //TRV position mode 
               }
               boolIsMCURequest = false;
              
               sAppUartEvent.eType_UartNextState = APP_E_EVENT_UART_IDLE;
          break;
               

          case GET_POSITION:
               memcpy(ui8ValvePositionChange,ui8ReadGetFlagBuffer, sizeof(ui8ValvePositionChange));
               SetValvePosition(ui8ValvePositionChange[1]);   //if valve position change then set valve and copy the data on pi heating demand attribute.
               boolIsMCURequest = false;
                sAppUartEvent.eType_UartNextState = APP_E_EVENT_UART_IDLE; 
          break;
          

          default:
          break;
       }
       memset(ui8ReadGetFlagBuffer,0,sizeof(ui8ReadGetFlagBuffer)); 
       boolMCUInterrupt = false; 
                  
   }
}



//function to reset all flags 
void ResetFlags(void)
{
  boolReady = false;
  boolSetCommand = false;
  boolSetTemperature = false;
  boolSetHostflags = false;
  boolSetPointChange = false;
}
/****************************************************************************
 *
 * NAME: vUART_STATE_StateMachine
 *
 * DESCRIPTION:
 * Check different states of Thermostat and RF
 *
 * RETURNS:
 * void
 *
****************************************************************************/
void UARTStateStateMachine(void)
{
   if (e_StateMachineThermostat == BOOTLOADER_STATE)
   {
        switch (sAppBootloaderEvent.eType_BootloaderState) 
        {			
            case APP_E_EVENT_UART_START_FIRMWARE_UPDATE:                                 
                    //bRestartBootloaderMode = FALSE;  did not get declaration
                  UARTStateStartFirmwareUpdate();  
            break;
            case APP_E_EVENT_UART_TRANSFER_FRIMWARE_UPDATE:		
                    //UARTStateTransferFirmwareUpdate(); This function is not created here
            break;
            case APP_E_EVENT_BOOTLOADER_STATE_IDLE:
            break;
            default:			
            break;
        }

   }
   else if (e_StateMachineThermostat == PRODCUTION_MODE)
   {

        switch (sAppProductionEvent.eType_ProductionState) 
        {
            case APP_E_EVENT_PRODUCTION_STATE:
                 GetMessageIdResponse(GET_PRODUCTION_MODE);  // send get production mode command 
            break;
            default:
            break;
                
        }
   }
   else if (e_StateMachineThermostat == RUNNING_STATE)
   {
            switch (sAppUartEvent.eType_UartNextState)
            {
              case APP_E_EVENT_UART_SEND_TEMPERATURE:
                     //UARTStateSendTemperature(u16SaveAttributeID);  TBD for replacement send set temperature command
              break;

              case APP_E_EVENT_UART_GET_TEMPERATURE:                    
                    GetMessageIdResponse(GET_TEMPERATURES); //send get temeperature command
                    boolReady = true; //set flag to execute communication state
                    boolIsMCURequest = true;
              break;

              case APP_E_EVENT_UART_GET_BATTERY:                     
                     GetMessageIdResponse(GET_BATTERY_STATUS); //send get battey status command
                     boolReady = true; //set flag to execute communication state
                     boolIsMCURequest = true;
//                     sAppUartEvent.eType_UartNextState = APP_E_EVENT_UART_IDLE;
              break;

              case APP_E_EVENT_UART_GET_COMPOSITE_DATA:                     
                      GetMessageIdResponse(GET_COMPOSITE_DATA1); //send get comosite data command
              break;

              case APP_E_EVENT_UART_ULP_WAS_TRIGGERED:                      
                      GetMessageIdResponse(GET_FLAGS);   //send get flag command
              break;

              case APP_E_EVENT_UART_SEND_FLAGS:                    
                   sAppUartEvent.eType_UartNextState = APP_E_EVENT_UART_IDLE;
              break;

              case APP_E_EVENT_UART_SEND_FLAGS_AUTO_MODE:
                      UARTStateSendFlagsAuto();     //send set flag with auto mode flag command
              break;

              case APP_E_EVENT_UART_SEND_FLAGS_MANU_MODE:
                      UARTStateSendFlagsManu();     //send set flag with manual mode flag command
              break;

              case APP_E_EVENT_UART_SEND_VALVE_POSITION:                      
                      UARTStateSendValvePosition();   //send set valve position command
              break;

              case APP_E_EVENT_UART_GET_VALVE_POSITION:                    
                      GetMessageIdResponse(GET_POSITION);   //send get valve position command
                     
              break;

              case APP_E_EVENT_UART_SEND_DEVICE_CONFIG:                      
//                      UARTStateSendDeviceConfig();          //send set device config command
//                      sAppUartEvent.eType_UartNextState = APP_E_EVENT_UART_IDLE; 
              break;

              case APP_E_EVENT_UART_SEND_DEVICE_CONFIG_2:                    
                      UARTStateSendDeviceConfig2();           //send set device config2 command   
              break;

              case APP_E_EVENT_UART_GET_DEVICE_CONFIG:                    
                      GetMessageIdResponse(GET_DEVICE_CONFIG);   //send get device config command
                      boolReady = true; //set flag to execute communication state
                      boolIsMCURequest = true;
//                      sAppUartEvent.eType_UartNextState = APP_E_EVENT_UART_IDLE;
              break;

              case APP_E_EVENT_UART_SEND_FLAGS_JOINED:                                           
                      UARTStateSendFlagsJoined();            //send set flag command with joined parameter
              break;

              case APP_E_EVENT_UART_SEND_FLAGS_LEAVE:                     
                      UARTStateSendFlagsLeave();             //send set flag command with leave ntw parameter
              break;

              case APP_E_EVENT_UART_SEND_FLAGS_RF_COMMUNICATION_OK:                     
                      UARTStateSendFlagsRFCommunicationOK();   //send set flag with RF communication okay command
              break;

              case APP_E_EVENT_UART_SEND_FLAGS_RF_COMMUNICATION_LOST:                      
                      UARTStateSendFlagsRFCommunicationLost();    //send set flag with RF communication lost command
              break;

              case APP_E_EVENT_UART_SEND_RTC:
                      UARTStateSendRTC();                    //send set RTC command 
              break;

              case APP_E_EVENT_UART_GET_RTC:
                      GetMessageIdResponse(GET_RTC);          //send get RTC command
              break;

              case APP_E_EVENT_UART_GET_FIRMWARE_VERSION:                     
                      GetMessageIdResponse(GET_FIRMWARE_VER);   //send get firmware version command
                      boolReady = true; //set flag to execute communication state
                      boolIsMCURequest = true;
//                      sAppUartEvent.eType_UartNextState = APP_E_EVENT_UART_IDLE;
              break;

              case APP_E_EVENT_UART_IDLE:
                   app_uart_close(); //close UART module
                     
              break;
              case APP_E_EVENT_UART_SEND_SYSTEM_MODE:
                     // vUART_STATE_SendDeviceConfig();    This is not created here
              break;

              case APP_E_EVENT_UART_GET_RTC_FROM_COORDINATOR:                     
                    //  vRTC_vRequestTimeFromCoordiantor();
              break;

              case APP_E_EVENT_UART_SEND_MASTER_RESET_REQUEST:
                      UARTStateMasterResetRequest();                 //send master reset with request flag command
              break;

              case APP_E_EVENT_UART_SEND_MASTER_RESET_SUCCESS:                     
                      UARTStateMasterResetSuccess();                //send master reset with rest flag command
              break;

              case APP_E_EVENT_UART_SEND_GET_PRODUCTION_MODE:
                      GetMessageIdResponse(GET_PRODUCTION_MODE);    //send get production mode command 
              break;

              case APP_E_EVENT_UART_SEND_SET_PRODUCTION_MODE_PASS:
                      UARTStateSendSetProductionModePass();    //send set production mode pass command
              break; 

              case APP_E_EVENT_UART_SEND_START_PRODUCTION_MODE:
                      UARTStateSendStartProductionMode();     //send set production mode start command
              break;

              case APP_E_EVENT_UART_LEAVE_NETWORK:                     
                   //   vUART_STATE_vLeaveNetwork();   This function not created here
              break;

              case APP_E_EVENT_UART_IDENTIFY_MODE:                     
              break;

              case APP_E_EVENT_UART_SEND_SWITCHPOINTS:
                    //UARTStateSendSwitchPoints();   //send switch point function. This function not created here
              break;


              default:
              break;
            }
   }

}

